From 2965a981b7ec22866fbdf2d567d87e2d068d3617 Mon Sep 17 00:00:00 2001
From: Thomas Bruederli <thomas@roundcube.net>
Date: Fri, 31 Jul 2015 16:04:08 -0400
Subject: [PATCH] Allow to search and import missing PGP pubkeys from keyservers using Publickey.js
---
program/lib/Roundcube/rcube_addressbook.php | 118 +++++++++++++++++++++++++++++++++++++++++++++++++++++------
1 files changed, 106 insertions(+), 12 deletions(-)
diff --git a/program/lib/Roundcube/rcube_addressbook.php b/program/lib/Roundcube/rcube_addressbook.php
index 6e2b439..31189e0 100644
--- a/program/lib/Roundcube/rcube_addressbook.php
+++ b/program/lib/Roundcube/rcube_addressbook.php
@@ -209,6 +209,7 @@
public function validate(&$save_data, $autofix = false)
{
$rcube = rcube::get_instance();
+ $valid = true;
// check validity of email addresses
foreach ($this->get_col_values('email', $save_data, true) as $email) {
@@ -216,12 +217,28 @@
if (!rcube_utils::check_email(rcube_utils::idn_to_ascii($email))) {
$error = $rcube->gettext(array('name' => 'emailformaterror', 'vars' => array('email' => $email)));
$this->set_error(self::ERROR_VALIDATE, $error);
- return false;
+ $valid = false;
+ break;
}
}
}
- return true;
+ // allow plugins to do contact validation and auto-fixing
+ $plugin = $rcube->plugins->exec_hook('contact_validate', array(
+ 'record' => $save_data,
+ 'autofix' => $autofix,
+ 'valid' => $valid,
+ ));
+
+ if ($valid && !$plugin['valid']) {
+ $this->set_error(self::ERROR_VALIDATE, $plugin['error']);
+ }
+
+ if (is_array($plugin['record'])) {
+ $save_data = $plugin['record'];
+ }
+
+ return $plugin['valid'];
}
/**
@@ -264,7 +281,8 @@
* @param array Assoziative array with save data
* Keys: Field name with optional section in the form FIELD:SECTION
* Values: Field value. Can be either a string or an array of strings for multiple values
- * @return boolean True on success, False on error
+ *
+ * @return mixed On success if ID has been changed returns ID, otherwise True, False on error
*/
function update($id, $save_cols)
{
@@ -294,8 +312,10 @@
/**
* Mark all records in database as deleted
+ *
+ * @param bool $with_groups Remove also groups
*/
- function delete_all()
+ function delete_all($with_groups = false)
{
/* empty for read-only address books */
}
@@ -515,24 +535,98 @@
$fn = join(' ', array($contact['surname'], $contact['firstname'], $contact['middlename']));
else if ($compose_mode == 1)
$fn = join(' ', array($contact['firstname'], $contact['middlename'], $contact['surname']));
- else
+ else if ($compose_mode == 0)
$fn = !empty($contact['name']) ? $contact['name'] : join(' ', array($contact['prefix'], $contact['firstname'], $contact['middlename'], $contact['surname'], $contact['suffix']));
+ else {
+ $plugin = rcube::get_instance()->plugins->exec_hook('contact_listname', array('contact' => $contact));
+ $fn = $plugin['fn'];
+ }
$fn = trim($fn, ', ');
- // fallback to display name
- if (empty($fn) && $contact['name'])
- $fn = $contact['name'];
-
- // fallback to email address
- if (empty($fn) && ($email = self::get_col_values('email', $contact, true)) && !empty($email)) {
- return $email[0];
+ // fallbacks...
+ if ($fn === '') {
+ // ... display name
+ if (!empty($contact['name'])) {
+ $fn = $contact['name'];
+ }
+ // ... organization
+ else if (!empty($contact['organization'])) {
+ $fn = $contact['organization'];
+ }
+ // ... email address
+ else if (($email = self::get_col_values('email', $contact, true)) && !empty($email)) {
+ $fn = $email[0];
+ }
}
return $fn;
}
/**
+ * Build contact display name for autocomplete listing
+ *
+ * @param array Hash array with contact data as key-value pairs
+ * @param string Optional email address
+ * @param string Optional name (self::compose_list_name() result)
+ * @param string Optional template to use (defaults to the 'contact_search_name' config option)
+ *
+ * @return string Display name
+ */
+ public static function compose_search_name($contact, $email = null, $name = null, $templ = null)
+ {
+ static $template;
+
+ if (empty($templ) && !isset($template)) { // cache this
+ $template = rcube::get_instance()->config->get('contact_search_name');
+ if (empty($template)) {
+ $template = '{name} <{email}>';
+ }
+ }
+
+ $result = $templ ?: $template;
+
+ if (preg_match_all('/\{[a-z]+\}/', $result, $matches)) {
+ foreach ($matches[0] as $key) {
+ $key = trim($key, '{}');
+ $value = '';
+
+ switch ($key) {
+ case 'name':
+ $value = $name ?: self::compose_list_name($contact);
+
+ // If name(s) are undefined compose_list_name() may return an email address
+ // here we prevent from returning the same name and email
+ if ($name === $email && strpos($result, '{email}') !== false) {
+ $value = '';
+ }
+
+ break;
+
+ case 'email':
+ $value = $email;
+ break;
+ }
+
+ if (empty($value)) {
+ $value = strpos($key, ':') ? $contact[$key] : self::get_col_values($key, $contact, true);
+ if (is_array($value)) {
+ $value = $value[0];
+ }
+ }
+
+ $result = str_replace('{' . $key . '}', $value, $result);
+ }
+ }
+
+ $result = preg_replace('/\s+/', ' ', $result);
+ $result = preg_replace('/\s*(<>|\(\)|\[\])/', '', $result);
+ $result = trim($result, '/ ');
+
+ return $result;
+ }
+
+ /**
* Create a unique key for sorting contacts
*/
public static function compose_contact_key($contact, $sort_col)
--
Gitblit v1.9.1